<?php
/**
 * iPay Standard Payment Gateway
 *
 * @author   Moses King'ori <moseskariuki2119@gmail.com>
 * @category Class
 * @package  Hotelier/Classes/Payment
 * @version  1.0.0
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

class HTL_Gateway_Ipay extends HTL_Payment_Gateway {

	/** @var boolean Whether or not logging is enabled */
	public static $log_enabled = false;

	/** @var HTL_Log Logger instance */
	public static $log = false;

	/**
	 * Get things going.
	 */
	public function __construct() {
		$this->id = 'ipay';
		$this->title = esc_html__('iPay', 'wp-hotelier');
		$this->description = htl_get_option('ipay_message');
		$this->icon = HTL()->plugin_url() . '/includes/gateways/ipay/assets/images/ipay.png';
		$this->testmode = htl_get_option('ipay_sandbox') ? true : false;
		$this->debug = htl_get_option('ipay_log') ? true : false;
		$this->email = $this->get_email();
		$this->merchantname = htl_get_option('ipay_merchantname');
		$this->vid = htl_get_option('ipay_vid');
		$this->hashkey = htl_get_option('ipay_hashkey');
		$this->autopay = htl_get_option('ipay_autopay') ? true : false;
		$this->merchantcountry = htl_get_option('ipay_merchantcountry');

		self::$log_enabled = $this->debug;

		include_once 'includes/class-htl-gateway-ipay-response.php';
		new HTL_Gateway_Ipay_Response($this);

		add_filter('hotelier_settings_payment', array($this, 'settings_fields'), 0);
	}

	/**
	 * Add settings fields in admin.
	 */
	public function settings_fields($fields) {
		$gateway_fields = array(
			'ipay_settings' => array(
				'id' => 'ipay_settings',
				'name' => '<strong>' . esc_html__('iPay settings', 'wp-hotelier') . '</strong>',
				'type' => 'header',
				'class' => 'htl-ui-row--section-description',
			),
			'ipay_description' => array(
				'id' => 'ipay_description',
				'desc' => sprintf(wp_kses(__('iPay standard sends customers to iPay to enter their payment information. iPay IPN requires <a href="%s">fsockopen/cURL</a> support to update bookings after payment.', 'wp-hotelier'), array('a' => array('href' => array()))), esc_url('?page=hotelier-settings&tab=tools')),
				'type' => 'description',
			),
			'ipay_message' => array(
				'id' => 'ipay_message',
				'name' => esc_html__('iPay description', 'wp-hotelier'),
				'desc' => esc_html__('The description the user sees during the booking.', 'wp-hotelier'),
				'std' => esc_html__('Pay with iPay - The safer, easier way to pay online!', 'wp-hotelier'),
				'type' => 'textarea',
			),
			'ipay_sandbox' => array(
				'id' => 'ipay_sandbox',
				'name' => esc_html__('iPay sandbox', 'wp-hotelier'),
				'desc' => esc_html__('Enable test mode.', 'wp-hotelier'),
				'subdesc' => esc_html__('While in test mode no live transactions are processed. To fully use test mode, you must have a iPay sandbox (test) account.', 'wp-hotelier'),
				'type' => 'checkbox',
				'toggle' => true,
			),
			'ipay_log' => array(
				'id' => 'ipay_log',
				'name' => esc_html__('Debug log', 'wp-hotelier'),
				'desc' => esc_html__('Enable logging.', 'wp-hotelier'),
				'subdesc' => sprintf(__('Log iPay events, such as IPN requests, inside <code>%s</code>. Please note: this may log personal information. We recommend using this for debugging purposes only and deleting the logs when finished.', 'wp-hotelier'), htl_get_log_file_path('ipay')),
				'type' => 'checkbox',
				'toggle' => true,
			),
			'ipay_merchantcountry' => array(
				'id' => 'ipay_merchantcountry',
				'name' => esc_html__('iPay merchant country', 'wp-hotelier'),
				'desc' => esc_html__('Select merchant country.', 'wp-hotelier'),
				'subdesc' => esc_html__('Enables sending of server to server callbacks.', 'wp-hotelier'),
				'options' => array(
					'ke' => 'Kenya',
					'tz' => 'Tanzania',
					'tg' => 'Togo',
					'ug' => 'Uganda',
				),
				'type' => 'select',
			),
			'ipay_merchantname' => array(
				'id' => 'ipay_merchantname',
				'name' => esc_html__('iPay Merchant Name', 'wp-hotelier'),
				'desc' => esc_html__('Enter the name that describes your business.', 'wp-hotelier'),
				'type' => 'text',
				'std' => '',
			),
			'ipay_vid' => array(
				'id' => 'ipay_vid',
				'name' => esc_html__('iPay Vendor ID', 'wp-hotelier'),
				'desc' => esc_html__('Enter your Vendor ID issued by iPay.', 'wp-hotelier'),
				'type' => 'text',
				'std' => '',
			),
			'ipay_hashkey' => array(
				'id' => 'ipay_hashkey',
				'name' => esc_html__('iPay Hashkey', 'wp-hotelier'),
				'desc' => esc_html__('Enter your Hashkey issued by iPay.', 'wp-hotelier'),
				'type' => 'text',
				'std' => '',
			),
			'ipay_autopay' => array(
				'id' => 'ipay_autopay',
				'name' => esc_html__('iPay Autopay', 'wp-hotelier'),
				'subdesc' => esc_html__('Enable server to server callback triggering.', 'wp-hotelier'),
				'type' => 'checkbox',
				'toggle' => true,
			),
		);

		return array_merge($fields, $gateway_fields);
	}

	/**
	 * Logging method
	 * @param  string $message
	 */
	public static function log($message) {
		if (self::$log_enabled) {
			if (empty(self::$log)) {
				self::$log = new HTL_Log();
			}

			self::$log->add('ipay', $message);
		}
	}

	/**
	 * Get valid PayPal email.
	 *
	 * @return string
	 */
	public function get_email() {
		$email = htl_get_option('paypal_email');

		if (is_email($email)) {
			return $email;
		} else {
			return '';
		}
	}

	/**
	 * Process the payment and return the result
	 *
	 * @param int $reservation_id
	 * @return array
	 */
	public function process_payment($reservation_id) {
		include_once 'includes/class-htl-gateway-ipay-request.php';

		$reservation = htl_get_reservation($reservation_id);
		$ipay_request = new HTL_Gateway_Ipay_Request($this);

		// Remove cart
		HTL()->cart->empty_cart();

		return array(
			'result' => 'success',
			'redirect' => $ipay_request->get_request_url($reservation, $this->testmode),
		);
	}
}
