<?php
/**
 * Handles responses from iPay.
 *
 * @author   Moses King'ori <moseskariuki2119@gmail.com>
 * @category Class
 * @package  Hotelier/Classes/Payment
 * @version  1.0.0
 */

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

if (!class_exists('HTL_Gateway_Ipay_Response')):

/**
 * HTL_Gateway_Ipay_Response Class
 */
	class HTL_Gateway_Ipay_Response {

		/** @var bool Sandbox mode */
		protected $sandbox = false;

		/** @var string Receiver email address to validate */
		protected $email;

		/**
		 * Constructor
		 */
		public function __construct($gateway) {
			add_action('hotelier_api_htl_gateway_ipay', array($this, 'check_response'));

			$this->gateway = $gateway;

			// $this->vid = $vid;
			// $this->sandbox = $sandbox;
		}

		/**
		 * Check for iPay IPN Response
		 */
		public function check_response() {

			if (empty($_GET)) {
				wp_die("Transaction could not be verified", "iPay IPN", array('response' => 500));
			}

			$ipn_base = "";

			switch ($this->gateway->merchantcountry) {
			case 'ke':
				$ipn_base = "https://www.ipayafrica.com/ipn";
				break;

			case 'tz':
				$ipn_base = "https://payments.elipa.co.tz/v3/tz/ipn";
				break;

			case 'ug':
				$ipn_base = "https://payments.elipa.co.ug/v3/ug/ipn";
				break;

			case 'tg':
				$ipn_base = "https://payments.elipa.tg/v1/tg/index.php/ipn/check";
				break;

			default:
				echo "Unknown country of operation";
				exit();
				break;
			}

			$val = $this->gateway->vid;
			$val1 = sanitize_text_field($_GET['id']);
			$val2 = sanitize_text_field($_GET['ivm']);
			$val3 = sanitize_text_field($_GET['qwh']);
			$val4 = sanitize_text_field($_GET['afd']);
			$val5 = sanitize_text_field($_GET['poi']);
			$val6 = sanitize_text_field($_GET['uyt']);
			$val7 = sanitize_text_field($_GET['ifd']);
			$ipnurl = $ipn_base . "?vendor=" . $val . "&id=" . $val1 . "&ivm=" . $val2 . "&qwh=" . $val3 . "&afd=" . $val4 . "&poi=" . $val5 . "&uyt=" . $val6 . "&ifd=" . $val7;

			$status = "";

			if ($this->gateway->testmode) {
				$status = "aei7p7yrx4ae34";
			} else {
				$fp = fopen($ipnurl, "rb");
				$status = stream_get_contents($fp, -1, -1);
				fclose($fp);

			}

			$payment_status = "";

			switch ($status) {
			case 'aei7p7yrx4ae34':
				$payment_status = 'completed';
				break;
			case 'fe2707etr5s4wq':
				$payment_status = 'failed';
				break;

			default:
				$payment_status = 'failed';
				break;
			}

			$reservation_id = $_GET['id'];
			$reservation_key = $_GET['p1'];

			if (!$reservation = htl_get_reservation($reservation_id)) {
				// We have an invalid $reservation_id
				$reservation_id = htl_get_reservation_id_by_reservation_key($reservation_key);
				$reservation = htl_get_reservation($reservation_id);
			}

			if (!$reservation) {
				HTL_Gateway_Ipay::log('Error: Reservation Keys do not match.');
				return false;
			}

			HTL_Gateway_Ipay::log('Found reservation #' . $reservation->id);
			HTL_Gateway_Ipay::log('Payment status: ' . $payment_status);

			if (method_exists($this, 'payment_status_' . $payment_status)) {
				call_user_func(array($this, 'payment_status_' . $payment_status), $reservation, $_GET);
			}

			$successpage = $this->gateway->get_return_url($reservation);

			header("location: $successpage");
			exit();
		}

		/**
		 * Handle a completed payment
		 * @param  HTL_Reservation $reservation
		 */
		protected function payment_status_completed($reservation, $posted) {

			HTL_Gateway_Ipay::log('Payment completed');

			$this->save_ipay_meta_data($reservation, $posted);
			$this->payment_complete($reservation, (!empty($posted['id']) ? sanitize_text_field($posted['id']) : ''), esc_html__('IPN payment completed', 'wp-hotelier'));
		}

		/**
		 * Handle a failed payment
		 * @param  HTL_Reservation $reservation
		 */
		protected function payment_status_failed($reservation, $posted) {
			$reservation->update_status('failed', sprintf(esc_html__('Payment %s via IPN.', 'wp-hotelier'), sanitize_text_field($posted['payment_status'])));
		}

		/**
		 * Save important data from the IPN to the reservation
		 * @param HTL_Reservation $reservation
		 */
		protected function save_ipay_meta_data($reservation, $posted) {
			if (!empty($posted['payment_type'])) {
				update_post_meta($reservation->id, 'Payment type', sanitize_text_field($posted['payment_type']));
			}
		}

		/**
		 * Get the reservation from the iPay 'Custom' variable
		 *
		 * @param  string $raw_custom JSON Data passed back by iPay
		 * @return bool|HTL_Reservation object
		 */
		protected function get_ipay_reservation($raw_custom) {

			// We have the data in the correct format, so get the reservation
			if (($custom = json_decode($raw_custom)) && is_object($custom)) {
				$reservation_id = $custom->reservation_id;
				$reservation_key = $custom->reservation_key;

				// Nothing was found
			} else {
				HTL_Gateway_Ipay::log('Error: Reservation ID and key were not found in "custom".');
				return false;
			}

			if (!$reservation = htl_get_reservation($reservation_id)) {
				// We have an invalid $reservation_id
				$reservation_id = htl_get_reservation_id_by_reservation_key($reservation_key);
				$reservation = htl_get_reservation($reservation_id);
			}

			if (!$reservation || $reservation->reservation_key !== $reservation_key) {
				HTL_Gateway_Ipay::log('Error: Reservation Keys do not match.');
				return false;
			}

			return $reservation;
		}

		/**
		 * Complete reservation and add transaction ID
		 * @param  HTL_Reservation $reservation
		 * @param  string $txn_id
		 * @param  string $note
		 */
		protected function payment_complete($reservation, $txn_id = '', $note = '') {
			$reservation->add_reservation_note($note);
			$reservation->payment_complete($txn_id);
		}
	}

endif;
